package net.sf.logdistiller.logtypes.http;

/*
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.io.*;
import java.text.ParseException;

import net.sf.logdistiller.LogEvent;
import net.sf.logdistiller.LogType;
import net.sf.logdistiller.util.StringCutter;

/**
 * Log event for commonly used CLF format of web servers, either
 * <a href="http://httpd.apache.org/docs/2.0/logs.html#common">Common Log Format</a>
 * or <a href="http://httpd.apache.org/docs/2.0/logs.html#combined">Combined Log Format</a>
 */
public class HttpClfLogEvent
    extends LogEvent
{
    public final static String ID = "http.CLF";

    public final String host;

    public final String identity;

    public final String userid;

    public final String time;

    public final String timeDate;

    public final String timeTime;

    public final String request;

    public final String requestMethod;

    public final String requestUri;

    public final String requestProtocol;

    public final String status;

    public final String size;

    public final String referer;

    public final String userAgent;

    public final String userAgentName;

    public final String userAgentVersion;

    public final String userAgentDetails;

    public final String other;

    public final static LogType LOGTYPE = new LogType.Basic( ID );

    private final static String[] ATTRIBUTE_NAMES = { "logSource", "host", "identity", "userid",
        "time", "time.date", "time.time",
        "request", "request.method", "request.uri", "request.protocol",
        "status", "size", "referer",
        "user-agent", "user-agent.name", "user-agent.version", "user-agent.details",
        "other" };

    public final static LogType.Description DESCRIPTION = new Description( (LogType.Basic) LOGTYPE, ATTRIBUTE_NAMES );

    public HttpClfLogEvent( Factory factory, String rawLog )
        throws ParseException
    {
        super( factory, rawLog );
        StringCutter sc = new StringCutter( rawLog );
        host = sc.parseTo( " " );
        identity = sc.parseTo( " " );
        userid = sc.parseTo( " [" );
        time = sc.parseTo( "] \"" );
        timeDate = time.substring( 0, 11 );
        timeTime = time.substring( 12, 20 );
        request = sc.parseTo( "\" " );
        StringCutter req = new StringCutter( request );
        requestMethod = req.parseTo( " " );
        requestUri = req.parseTo( " " );
        requestProtocol = req.getRemaining();
        status = sc.parseTo( " " );
        size = sc.parseTo( " \"", false );
        if ( sc.getRemaining().length() == 0 )
        { // Common Log Format only
            referer = userAgent = userAgentName = userAgentVersion = userAgentDetails = other = "";
        }
        else
        { // Combined Log Format
            referer = sc.parseTo( "\" \"" );
            userAgent = sc.parseTo( "\"" );
            StringCutter ua = new StringCutter( userAgent );
            userAgentName = ua.parseTo( "/", false );
            userAgentVersion = ua.parseTo( " ", false );
            ua.parseTo( "(", false );
            userAgentDetails = ua.parseTo( ")", false );
            other = sc.getRemaining().trim();
        }
        setAttributes( new String[] { factory.getLogSource(), host, identity, userid,
            time, timeDate, timeTime,
            request, requestMethod, requestUri, requestProtocol,
            status, size, referer,
            userAgent, userAgentName, userAgentVersion, userAgentDetails,
            other } );
    }

    private static class Description
        extends LogType.Description
    {
        public Description( LogType.Basic logtype, String[] attributeNames )
        {
            super( logtype, attributeNames );
            logtype.setDescription( this );
        }

        public LogEvent.Factory newFactory( Reader reader, String logSource )
            throws IOException
        {
            return new Factory( this, reader, logSource );
        }

        public int getTimestampAttribute()
        {
            return 3;
        }

        public String getDefaultSamplingAttributes()
        {
            return "request.uri,user-agent";
        }
    }

    private static class Factory
        extends LogEvent.Factory
    {
        private final LineNumberReader reader;

        public Factory( Description description, Reader reader, String logSource )
            throws FileNotFoundException
        {
            super( description, logSource );
            this.reader = new LineNumberReader( reader );
        }

        protected LogEvent readNextEvent()
            throws IOException, ParseException
        {
            String curLine = reader.readLine();
            if ( curLine == null )
            {
                // EOF
                return null;
            }
            try
            {
                return new HttpClfLogEvent( this, curLine );
            }
            catch ( RuntimeException re )
            {
                throw new RuntimeException( "error while parsing line '" + curLine + "'", re );
            }
        }
    }
}
